;;;
;;; mem.h
;;;
;;; created: <Fri May  7 21:09:58 1999>
;;; last modified: <Fri Oct  5 23:22:00 2001>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the memory functions in libASM.a
;;;

;;;*
;;;* A short discription of libASM's memory allocation routines
;;;*
;;;* allocated chunk:
;;;*
;;;*   chunk -> +-------------------------------------------------------------+
;;;*   (page    | Size of previous chunk, 0 if first chunk (dword)            |
;;;*    aligned)+-------------------------------------------------------------+
;;;*            | Size of chunk in bytes (dword)                              |
;;;*            +-------------------------------------------------------------+
;;;*            | Flags (dword):                                              |
;;;*            |  0xcf000000                                                 |
;;;*            |    ||||||||                                                 |
;;;*            |    ||||++++-----> NUL                                       |
;;;*            |    |||+---------> mprotect                                  |
;;;*            |    ||+----------> 0001b this is the first chunk             |
;;;*            |    ||             0010b this is a free chunk                |
;;;*            |    ||             0100b this is the last chunk              |
;;;*            |    ||             1000b forbid other jobs to modify chunk   |
;;;*            |    ||                   (not default)                       |
;;;*            |    ++-----------> Identification (0xcf)                     |
;;;*            +-------------------------------------------------------------+
;;;*            | PID of the job that created this chunk.                     |
;;;*            +-------------------------------------------------------------+
;;;*            | Proc that will be called by MemoryFree instead of releasing |
;;;*            | this chunk.                                                 |
;;;*            +-------------------------------------------------------------+
;;;*            | Pad                                                         |
;;;*     mem -> +-------------------------------------------------------------+
;;;*            | User data....                                               |
;;;*            :                                                             :
;;;*            .                                                             .
;;;*            .                                                             .
;;;*            +-------------------------------------------------------------+
;;;*             Next chunk...
;;;*
;;;* free chunk
;;;*   chunk -> +-------------------------------------------------------------+
;;;*   (page    | Size of previous chunk, 0 if first chunk (dword)            |
;;;*    aligned)+-------------------------------------------------------------+
;;;*            | Size of chunk in bytes (dword)                              |
;;;*            +-------------------------------------------------------------+
;;;*            | Flags (dword): free, PROT_NONE                              |
;;;*            +-------------------------------------------------------------+
;;;*            | NULL                                                        |
;;;*            +-------------------------------------------------------------+
;;;*            | NULL                                                        |
;;;*            +-------------------------------------------------------------+
;;;*            | Pad                                                         |
;;;*     mem -> +-------------------------------------------------------------+
;;;*            | User data.... (0...)                                        |
;;;*            :                                                             :
;;;*            .                                                             .
;;;*            .                                                             .
;;;*            +-------------------------------------------------------------+
;;;*             Next chunk...
;;;*

%ifndef _mem_h_
%define _mem_h_

;;;*
;;;* The _memory_chunk structure defines the chunk headers layout:
;;;*
;;;* .prev:	size of previous chunk
;;;* .size:	size of chunk
;;;* .flags:	some flags
;;;* .pid:	pid of owner process
;;;* .proc:	proc that has to be called instead of MemoryFree (NULL means
;;;*		to call MemoryFree)
;;;*

;;;
struc _memory_chunk
.prev:		resd 1		; size of previous chunk
.size:		resd 1		; size of chunk
.flags:		resd 1		; some flags
.pid:		resd 1		; pid of owner process
.proc:		resd 1		; proc that has to be called instead of MemoryFree
		resd 3		; Hand padded for optimal access speed
endstruc

%ifdef ___libASM___
%macro MemoryLOCK 0
%%Loop:	cmp [_memory_LOCK],byte false
	je %%NoLoop
	%include "sys.h"
	Sleep dword 0, dword 2
	jmp short %%Loop
%%NoLoop:
	mov [_memory_LOCK],byte true
%endm

%define MemoryUNLOCK	mov [_memory_LOCK],byte false

%macro MemoryHAS_INIT 0	; 1
	cmp [_memory_HAS_INIT],byte true
	je %%Goon
	call _MemoryInit
%%Goon:
%endm
%endif	; ___libASM___

;;;*
;;;* The flags are or'd combinations out of these:
;;;*
;;;* _memory_identify_flag:	This flag sets the identification of a chunk.
;;;* _memory_firstchunk_flag:	This marks the first chunk.
;;;* _memory_lastchunk_flag:	This marks the last chunk.
;;;* _memory_freechunk_flag:	This defines the chunk to be unused.
;;;* _memory_forbidmodify_flag:	This allows only the thread with pid=[.pid] to
;;;*				modify the chunk.
;;;* _memory_PROT_READ:		Mark chunk as readable
;;;* _memory_PROT_WRITE:		Mark chunk as writeable
;;;* _memory_PROT_EXEC:		Mark chunk as executeable
;;;* _memory_PROT_NONE:		Forbid any access
;;;*

%define _memory_flag_IDshift 24
%define _memory_flag_IDmask 0xff << _memory_flag_IDshift
%define _memory_identify_flag 0xcf << _memory_flag_IDshift
%define _memory_flag_shift 20
%define _memory_flag_mask 0xf << _memory_flag_shift
%define _memory_firstchunk_flag 0001b << _memory_flag_shift
%define _memory_lastchunk_flag 0010b << _memory_flag_shift
%define _memory_freechunk_flag 0100b << _memory_flag_shift
%define _memory_forbidmodify_flag 1000b << _memory_flag_shift
%define _memory_PROT_shift 16
%define _memory_flag_MPROTmask 0xf << _memory_PROT_shift
%define _memory_PROT_READ PROT_READ << _memory_PROT_shift
%define _memory_PROT_WRITE PROT_WRITE << _memory_PROT_shift
%define _memory_PROT_EXEC PROT_EXEC << _memory_PROT_shift
%define _memory_PROT_NONE PROT_NONE << _memory_PROT_shift
%define _memory_NUL_shift 0
%define _memory_NUL_mask 0xffff << _memory_NUL_shift
%define _memory_NUL_flag 0 << _memory_NUL_shift


;;;*
;;;* MemoryInit
;;;*
;;;*  In: MemoryInit
;;;*  Out: eax=-1 (on error); 0 (on success)
;;;*
;;;*  Initialize the memory routines.
;;;*
%macro MemoryInit 0
	extern _MemoryInit
	call _MemoryInit
%endm

;;;*
;;;* MemoryAllocate
;;;*
;;;*  In: MemoryAllocate size (eax)
;;;*  Out: eax=ptr (on success); -1 (on error)
;;;*
;;;*  Allocate some memory.
;;;*
%macro MemoryAllocate 1
	extern _MemoryAllocate
%ifnidni %1,eax
	mov eax,%1
%endif
	call _MemoryAllocate
%endm

;;;*
;;;* MemoryResize
;;;*
;;;*  In: MemoryResize ptr, newsize (ptr [eax+0], newsize [eax+4])
;;;*  Out: eax=(new) ptr (on success); -1 (on error)
;;;*
;;;*  Change size of allocated chunk
;;;*  You should resave your pointer after using this rout, 'cause it might
;;;*  change. The data will be moved, so don't worry, be happy!
;;;*
%macro MemoryResize 1-2
	extern _MemoryResize
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _MemoryResize
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;*
;;;* MemoryFree
;;;*
;;;*  In: MemoryFree ptr (eax)
;;;*  Out: eax=0 (on success); -1 (on error)
;;;*
;;;*  Give an allocated block back to the system.
;;;*
%macro MemoryFree 1
	extern _MemoryFree
%ifnidni %1,eax
	mov eax,%1
%endif
	call _MemoryFree
%endm

;;;*
;;;* MemoryTune
;;;*
;;;*  In: MemoryTune
;;;*  Out: -
;;;*
;;;*  Tune the chunk structure.
;;;*
%macro MemoryTune 0
	extern _MemoryTune
	call _MemoryTune
%endm

;;;*
;;;* MemoryProtect
;;;*
;;;*  In: MemoryProtect ptr, flags (ptr [eax+0], flags [eax+4])
;;;*  Out: 0 (success), -1 (error)
;;;*
;;;*  NOTE: This changes the protection for the hole chunk. If You want to change
;;;*  only a part of it, you can do this via sys_mprotect. Maybe that the lib code
;;;*  will fail or will change the protection. You have been warned.
;;;*
%macro MemoryProtect 1-2
	extern _MemoryProtect
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _MemoryProtect
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;*
;;;* MemoryGetsize
;;;*
;;;*  In: MemoryGetsize ptr (eax)
;;;*  Out: eax=Size (on success); -1 (on error)
;;;*
;;;*  Get size of chunk.
;;;*
%macro MemoryGetsize 1
	extern _MemoryGetsize
%ifnidni %1,eax
	mov eax,%1
%endif
	call _MemoryGetsize
%endm

;;;*
;;;* MemoryPrintChunkStruc
;;;*
;;;*  In: MemoryPrintChunkStruc
;;;*  Out: -
;;;*
;;;*  Print chunkstructure to STDOUT.
;;;*
%macro MemoryPrintChunkStruc 0
	extern _MemoryPrintChunkStruc
	call _MemoryPrintChunkStruc
%endm


;;;*
;;;* MemorySetProc
;;;*
;;;*  In: MemorySetProc ptr, proc (ptr [eax+0], proc [eax+4])
;;;*  Out: 0 (success), -1 (error)
;;;*
;;;*  This sets the pointer to a procedure that will be called instead of
;;;*  releasing the chunk. If proc is NULL nothing will be called. This might
;;;*  be usefull when working width threads. Before libASM can release the
;;;*  chunk, proc has to be set back to NULL. proc is called with the same
;;;*  register setup which was used to call MemoryFree.
;;;*
%macro MemorySetProc 1-2
	extern _MemorySetProc
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _MemorySetProc
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* MemorySetPid
;;;*
;;;*  In: MemorySetPid ptr, pid (ptr [eax+0], pid [eax+4])
;;;*  Out: 0 (success), -1 (error)
;;;*
;;;*  This sets the (current) pid in the header of the chunk refered by ptr
;;;*  (if allowed).
;;;*
%macro MemorySetPid 1-2
	extern _MemorySetPid
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _MemorySetPid
%if %0 = 2
	add esp,byte 8
%endif
%endm

%endif	; _mem_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: