﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace VizBin
{
    abstract class AbstractImageProcessor : ImageProcessor
    {

		/// <summary>
		/// Returns the name of ImageProcessor. Should be a descriptive name as this will be presented to
		/// the user.
		/// </summary>
		/// <returns>String name of the processor</returns>
		abstract public String getName();

		/// <summary>
		/// Create an image with the specified dimensions. This method proxies the results to
		/// the createImage method after checking bounds and other simple checks to ensure
		/// the parameters are valid. ArgumentExceptions are thrown if:
		/// 1) vab_bytes is null or has a length less than 1
		/// 2) vn_width is less than 1
		/// 3) vn_height is less than 1
		/// </summary>
		/// <param name="vab_bytes">byte[] array to paint into a bitmap</param>
		/// <param name="vn_width">int width of the bitmap to create</param>
		/// <param name="vn_height">int height of the bitmap to create</param>
		/// <returns>a bitmap</returns>
		internal Bitmap getImage(byte[] vab_bytes, int vn_width, int vn_height)
		{
			if (vab_bytes == null)
				throw new ArgumentException("Byte array is null");
			else if (vab_bytes.Length < 1)
				throw new ArgumentException("Byte array is empty");
			else if (vn_width < 1)
				throw new ArgumentException("Width dimension is less than 1");
			else if (vn_height < 1)
				throw new ArgumentException("Height dimension is less than 1");
			else
				return createImage(vab_bytes, vn_width, vn_height);
		}

		/// <summary>
		/// Create an image with the specified dimensions. This method proxies the results to
		/// the createOverviewImage method after checking bounds and other simple checks to ensure
		/// the parameters are valid. ArgumentExceptions are thrown if:
		/// 1) vab_bytes is null or has a length less than 1
		/// 2) vn_width is less than 1
		/// 3) vn_height is less than 1
		/// </summary>
		/// <param name="vab_bytes">byte[] array to paint into a bitmap</param>
		/// <param name="vn_width">int width of the bitmap to create</param>
		/// <param name="vn_height">int height of the bitmap to create</param>
		/// <returns>a bitmap</returns>
		internal Bitmap getOverviewImage(byte[] vab_bytes, int vn_width, int vn_height)
		{
			if (vab_bytes == null)
				throw new ArgumentException("Byte array is null");
			else if (vab_bytes.Length < 1)
				throw new ArgumentException("Byte array is empty");
			else if (vn_width < 1)
				throw new ArgumentException("Width dimension is less than 1");
			else if (vn_height < 1)
				throw new ArgumentException("Height dimension is less than 1");
			else
				return createOverviewImage(vab_bytes, vn_width, vn_height);
		}

		/// <summary>
		/// Return an Bitmap given a byte array, width, and height of the image. See the getImage method
		/// for the checks that will have been performed on the arguments.
		/// </summary>
		/// <param name="vab_bytes">byte[] array</param>
		/// <param name="vn_width">width</param>
		/// <param name="vn_height">height</param>
		/// <returns>Bitmap</returns>
		abstract public Bitmap createImage( byte[] vab_bytes, int vn_width, int vn_height );

		/// <summary>
		/// Return an Bitmap given a byte array, width, and height of the image. See the getImage method
		/// for the checks that will have been performed on the arguments.
		/// </summary>
		/// <param name="vab_bytes">byte[] array</param>
		/// <param name="vn_width">width</param>
		/// <param name="vn_height">height</param>
		/// <returns>Bitmap</returns>
		abstract public Bitmap createOverviewImage(byte[] vab_bytes, int vn_width, int vn_height);

        /// <summary>
        /// Returns the scaling factor that could be used to enlarge rendered images
        /// to use the total available image space.
        /// </summary>
        /// <param name="vn_width">the width of the total available space</param>
        /// <param name="vn_height">the height of the total available space</param>
        /// <param name="vn_bytes">the number of bytes to render</param>
        /// <returns>the scaling factor</returns>
        internal static int getScalingFactor(int vn_width, int vn_height, int vn_bytes)
        {
            if (vn_bytes < 0)
                throw new ArgumentException("Bytes must be greater than 0");
            if (vn_height < 0 || vn_height > 65534)
                throw new ArgumentException("Height must be between 1 and 65534");
            if( vn_width < 0 || vn_width > 65534 )
                throw new ArgumentException("Width must be between 1 and 65534");

            int n_zoom = 1;
            int n_total_bytes = vn_height * vn_width;
            if (n_total_bytes > vn_bytes && vn_bytes > 0)
                n_zoom = n_total_bytes / vn_bytes;
            return n_zoom;
        }
    }
}
