﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace VizBin
{
	class ByteFrequencyProcessor : AbstractImageProcessor
	{

		// the name of this image rendering engine
		private const String XS_NAME = "Byte Frequency";

		public override Bitmap createImage(byte[] vab_bytes, int vn_width, int vn_height)
		{
			return ByteFrequencyProcessor.createImage(vab_bytes, vn_width, vn_height, false);
		}

		public override Bitmap createOverviewImage(byte[] vab_bytes, int vn_width, int vn_height)
		{
			return ByteFrequencyProcessor.createImage(vab_bytes, vn_width, vn_height, true);
		}

		/// <summary>
		/// Given a byte array, and the dimensions of an image to be created, generate a bitmap
		/// (Image) with those dimenssions and generate the image. In our case, we will just color
		/// the respective pixels which represent the bits in the byte array.
		/// 
		/// Any classes which wish to be called from the controller must follow the same template
		/// as this class.
		/// </summary>
		/// <param name="vab_bytes">byte array</param>
		/// <param name="vn_width">width of the image</param>
		/// <param name="vn_height">height of the image</param>
		/// <param name="vb_is_overview">true if this is an imagemap false otherwise</param>
		/// <returns></returns>
		private static Bitmap createImage(byte[] vab_bytes, int vn_width, int vn_height, bool vb_is_overview)
		{
			Bitmap o_image = null;
			if (vn_width > 65535 || vn_width < 1)
				throw new ArgumentException("Width cannot be greater than 65534 and less than 1");
			else if (vn_height > 65535 || vn_width < 1)
				throw new ArgumentException("Height cannot be greater than 65534 and less than 1");
			else if (vab_bytes == null || vab_bytes.Length < 1)
				throw new ArgumentException("byte array cannot be null or have a length of 0");
			o_image = new Bitmap(vn_width, vn_height);
			// vars needed for image creation
			int x = 0;
			int y = 0;

			// if the number of bytes given is less than we need to create a full size
			// image, perform a 'zoom' function by finding a rough estimate number of 
			// bytes required to fill the image and represent all of the bytes
			int n_zoom = AbstractImageProcessor.getScalingFactor(vn_width, vn_height, vab_bytes.Length);

			// first iteration will count the number of bytes of each value in the image
			IDictionary<Byte, Int32> o_count_map = new Dictionary<Byte, Int32>();
			int n_largest_count = 1;
			foreach (byte chomp in vab_bytes)
				if (o_count_map.ContainsKey(chomp))
				{
					int n_val = o_count_map[chomp];
					n_val++;
					if (n_val > n_largest_count)
						n_largest_count = n_val;
					o_count_map[chomp] = n_val;
				}
				else
				{
					o_count_map.Add(chomp, 1);
				}

			// generate the color map
			IDictionary<Int32, Color> o_color_map = new Dictionary<Int32, Color>(n_largest_count);
			double n_gradient = 255 / n_largest_count;
			if (n_gradient == 0)
				n_gradient = 255 % n_largest_count;

			// this is our highest value, 16581375 -- dont go over it
			// coloring scheme black/blue -> occurs least
			// green -> medium frequency
			// red -> most frequently
			int n_red = 0;
			int n_green = 0;
			int n_blue = 0;
			for (int i = 1; i <= n_largest_count; i++)
			{
				if (n_blue < 255)
				{
					n_blue++;
				}
				else if( n_green < 255 )
				{
					n_blue = 0;
					n_green++;
				}
				else if (n_red < 255)
				{
					n_blue = 0;
					n_green = 0;
					n_red++;
				}
				o_color_map.Add(i, Color.FromArgb(n_red, n_green, n_blue));
			}

			// the loop will be broken out of when we reach the end of the byte array
			foreach (byte buffer in vab_bytes)
			{
				// get the color
				int n_color_index = o_count_map[buffer];
				Color o_curr_color = o_color_map[n_color_index];

				n_zoom = 1;
				for (int i = 0; i < n_zoom; i++)
				{
					// if we're at the end of the line, wrap to the next
					if (x + 1 >= vn_width)
					{
						x = 0;
						y++;
					}
					else
					{
						x++;
					}
					o_image.SetPixel(x, y, o_curr_color);
				}
			}//endfor
			return o_image;
		}

		/// <summary>
		/// Returns the name of this image processor, in this case "Default Image Processor"
		/// </summary>
		/// <returns>the name of this image processor</returns>
		public override string getName()
		{
			return XS_NAME;
		}
	}
}
