﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using VizBin;
using VizBin.searcher;

namespace VizBin.util
{
	class MagicFileParser
	{
		// private lazy init var to save on processing and resources
		private static MagicFileParser XO_PARSER = new MagicFileParser();
        // the records we currently have 
        private ICollection<MagicFileRecord> IO_RECORDS = new LinkedList<MagicFileRecord>();

		/// <summary>
		/// Private constructor preventing others from calling this class's constructor
		/// </summary>
		private MagicFileParser()
		{
			// open a stream reader to read the magic file which SHOULD exist in the same dir as the exe
			// make sure it does get deployed with it!
			if (File.Exists(".\\magic"))
			{
				try
				{
					String[] as_lines = File.ReadAllLines(".\\magic");
					foreach (String s_line in as_lines)
						this.parse(s_line);
					//using (StreamReader o_sr = new StreamReader(".\\magic"))
					//{

					//    String s_line = null;
					//    while ((s_line = o_sr.ReadLine()) != null)
					//    {
					//        this.parse(s_line);
					//    }
					//}
				}
				catch (Exception vo_e)
				{
					Controller.displayErrorMessage(vo_e.Message + "\n" + vo_e.Data + "\n" + vo_e.StackTrace);
				}
			}
		}

		/// <summary>
		/// Return the static instance of the parser. This will ensure we only do the resource intensive processing
		/// once and only once.
		/// </summary>
		/// <returns>MagicFileParser</returns>
		internal static MagicFileParser getParser()
		{
			return XO_PARSER;
		}

        /// <summary>
        /// Given the byte array, parse it for the records 
        /// </summary>
        /// <param name="vab_bytes">the byte array to search</param>
        /// <returns></returns>
        internal ICollection<SearchResult> parse(byte[] vab_bytes)
        {
            if( vab_bytes == null || vab_bytes.Length < 1 )
                throw new ArgumentException("Byte array cannot be null or empty");

            ICollection<SearchResult> o_results = new LinkedList<SearchResult>();
            int count = 0;
            foreach( MagicFileRecord o_rec in this.IO_RECORDS )
            {
                count++;
                switch( o_rec.Type )
                {
                    case MagicFileRecord.RecordType.BEDATE:
                    case MagicFileRecord.RecordType.BELONG:
                    case MagicFileRecord.RecordType.BESHORT:
						// this space for rent
                        break;
                    case MagicFileRecord.RecordType.BYTE:
                    case MagicFileRecord.RecordType.UBYTE:
						o_results.Concat(MagicFileParser.getResults(o_rec.SearchByte, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                    case MagicFileRecord.RecordType.DATE:
                    case MagicFileRecord.RecordType.LEDATE:
                        byte[] ab_bytes = BitConverter.GetBytes(o_rec.SearchDate.ToBinary());
						o_results.Concat(MagicFileParser.getResults(ab_bytes, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                    case MagicFileRecord.RecordType.LDATE:
                    case MagicFileRecord.RecordType.LELDATE:
                        ab_bytes = BitConverter.GetBytes(o_rec.SearchDate.ToBinary());
						o_results.Concat(MagicFileParser.getResults(ab_bytes, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                    case MagicFileRecord.RecordType.LELONG:
                    case MagicFileRecord.RecordType.LONG:
                        ab_bytes = BitConverter.GetBytes(o_rec.SearchLong);
						o_results.Concat(MagicFileParser.getResults(ab_bytes, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                    case MagicFileRecord.RecordType.LESHORT:
                    case MagicFileRecord.RecordType.SHORT:
                        ab_bytes = BitConverter.GetBytes(o_rec.SearchShort);
						o_results.Concat(MagicFileParser.getResults(ab_bytes, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                    case MagicFileRecord.RecordType.REGEX:
                    case MagicFileRecord.RecordType.SEARCH:
                        // this space for rent
                        break;
                    case MagicFileRecord.RecordType.STRING:
                    case MagicFileRecord.RecordType.USTRING:
                        char[] ac_chars = o_rec.SearchString.ToCharArray();
                        ab_bytes = new byte[ac_chars.Length];
                        for (int i = 0; i < ac_chars.Length; i++)
                            ab_bytes[i] = Convert.ToByte(ac_chars[i]);
						o_results.Concat(MagicFileParser.getResults(ab_bytes, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                    case MagicFileRecord.RecordType.UBELONG:
                    case MagicFileRecord.RecordType.UBESHORT:
						// this space for rent
                        break;
                    case MagicFileRecord.RecordType.ULELONG:
                    case MagicFileRecord.RecordType.ULONG:
                        ab_bytes = BitConverter.GetBytes(o_rec.SearchULong);
						o_results.Concat(MagicFileParser.getResults(ab_bytes, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                    case MagicFileRecord.RecordType.ULESHORT:
                    case MagicFileRecord.RecordType.USHORT:
                        ab_bytes = BitConverter.GetBytes(o_rec.SearchUShort);
						o_results.Concat(MagicFileParser.getResults(ab_bytes, o_rec.Message, vab_bytes, o_rec.ComparitorType));
                        break;
                }
            }
            return o_results;
        }

        /// <summary>
        /// Does an actual binary search on an array given the array, a message to add to the search result, and 
        /// the object to search for.
        /// </summary>
        /// <param name="vb_byte">the byte to look for</param>
        /// <param name="vs_message">the message to insert into the result</param>
        /// <param name="vab_bytes">the byte array to search</param>
		/// <param name="ve_comp_type">the MagicFile Comparitor type</param>
        /// <returns>an ICollection of SearchResults</returns>
        /// <exception cref="ArgumentException">thrown if any argument is null or the array is empty</exception>
        private static ICollection<SearchResult> getResults(byte vb_byte, String vs_message, byte[] vab_bytes, MagicFileRecord.CompareType ve_comp_type)
        {
            if (vab_bytes == null || vab_bytes.Length < 1) throw new ArgumentException("Byte array cannot be null or empty");
            if( vs_message == null ) throw new ArgumentException("Message cannot be null");

			ICollection<Int32> o_indicies = MagicFileParser.getResults(vb_byte, vab_bytes, ve_comp_type);
            ICollection<SearchResult> o_results = new LinkedList<SearchResult>();
            foreach (Int32 n_idx in o_indicies)
                o_results.Add(new SearchResult(vs_message, n_idx));
            return o_results;
        }

        /// <summary>
        /// Do a search for the specified byte in the given byte array. Return an ICollection of Int32's 
        /// which represent the index of each instance of the given byte. If there are no matches, a Collection
        /// of size 0 is returned.
        /// </summary>
        /// <param name="vb_byte">byte to search for</param>
        /// <param name="vab_bytes">byte array to search</param>
		/// <param name="ve_comp_type">Comparitor type for byte comparisons from the magic file record</param>
        /// <returns>ICollection of Int32 of size 0 or greater</returns>
        private static ICollection<Int32> getResults(byte vb_byte, byte[] vab_bytes, MagicFileRecord.CompareType ve_comp_type)
        {
            if (vab_bytes == null || vab_bytes.Length < 1) throw new ArgumentException("Byte array cannot be null or empty");
			int[] an_indicies = new int[vab_bytes.Length];
			int n_index = 0;
			int n_counter = 0;
			PredicateWrapper p = new PredicateWrapper( vb_byte, ve_comp_type );
			do
			{
				n_index = Array.FindIndex<byte>( vab_bytes, n_index, p.Matches);
				if (n_index > -1)
				{
					an_indicies[n_counter] = n_index;
					n_counter++;
					n_index++;
				}
			}
			while (n_index > -1);
			Array.Resize<int>(ref an_indicies, n_counter+1);
			return an_indicies;
        }
		private class PredicateWrapper
		{
			private byte b = 0;
			private MagicFileRecord.CompareType ie_comp_type = MagicFileRecord.CompareType.EQ;
			internal PredicateWrapper(byte vb, MagicFileRecord.CompareType ve_comp_type )
			{
				this.b = vb;
				this.ie_comp_type = ve_comp_type;
			}
			internal bool Matches(byte vb)
			{
				bool b_retval = false;
				switch (ie_comp_type)
				{
					case MagicFileRecord.CompareType.ANY:
					case MagicFileRecord.CompareType.CLEAR_BITS:
					case MagicFileRecord.CompareType.NOT:
					case MagicFileRecord.CompareType.SET_BITS:
						b_retval = false;
						break;
					case MagicFileRecord.CompareType.EQ:
						b_retval = (this.b == vb);
						break;
					case MagicFileRecord.CompareType.GT:
						b_retval = (this.b > vb);
						break;
					case MagicFileRecord.CompareType.LT:
						b_retval = (this.b < vb);
						break;
				}
				return b_retval;
			}
		}

        /// <summary>
        /// Does an actual binary search on an array given the array, a message to add to the search result, and 
        /// the object to search for.
        /// </summary>
        /// <param name="vab_bytes_to_find">the bytes to look for</param>
        /// <param name="vs_message">the message to insert into the result</param>
        /// <param name="vab_bytes_to_search">the byte array to search</param>
		/// <param name="ve_comp_type">the MagicFile ComparitorType to search with</param>
        /// <returns>an ICollection of SearchResults</returns>
        /// <exception cref="ArgumentException">thrown if any argument is null or the array is empty</exception>
		private static ICollection<SearchResult> getResults(byte[] vab_bytes_to_find, String vs_message, byte[] vab_bytes_to_search, MagicFileRecord.CompareType ve_comp_type)
        {
            if (vab_bytes_to_find == null || vab_bytes_to_find.Length < 1) throw new ArgumentException("Bytes to find cannot be null or empty");
            if (vab_bytes_to_search == null || vab_bytes_to_search.Length < 1) throw new ArgumentException("Bytes to search cannot be null or empty");

            byte b_first_byte = vab_bytes_to_find[0];
            ICollection<Int32> o_results = MagicFileParser.getResults(b_first_byte, vab_bytes_to_search, ve_comp_type);
            ICollection<SearchResult> o_rez = new LinkedList<SearchResult>();
            foreach (Int32 n_idx in o_results)
            {
				bool b_matched = (n_idx + vab_bytes_to_find.Length) < vab_bytes_to_search.Length;
				if (b_matched)
				{
					// copy the marked section of the array to a new array which makes it easier for searching
					byte[] ab_test_array = new byte[vab_bytes_to_find.Length];
					Array.Copy(vab_bytes_to_search, n_idx, ab_test_array, 0, vab_bytes_to_find.Length);

					// for each byte in both arrays make sure they match, break on the first non-match
					switch (ve_comp_type)
					{
						case MagicFileRecord.CompareType.ANY:
						case MagicFileRecord.CompareType.CLEAR_BITS:
						case MagicFileRecord.CompareType.NOT:
						case MagicFileRecord.CompareType.SET_BITS:
							break;
						case MagicFileRecord.CompareType.EQ:
							for (int i = 0; i < ab_test_array.Length && b_matched; i++)
								b_matched = b_matched && (ab_test_array[i] == vab_bytes_to_find[i]);
							break;
						case MagicFileRecord.CompareType.GT:
							for (int i = 0; i < ab_test_array.Length && b_matched; i++)
								b_matched = b_matched && (ab_test_array[i] > vab_bytes_to_find[i]);
							break;
						case MagicFileRecord.CompareType.LT:
							for (int i = 0; i < ab_test_array.Length && b_matched; i++)
								b_matched = b_matched && (ab_test_array[i] < vab_bytes_to_find[i]);
							break;
					}

					// if the test array and the search for array match, then add a new result
					if (b_matched)
						o_rez.Add(new SearchResult(vs_message, n_idx));
				}
            }
            return o_rez;
        }

		/// <summary>
		/// Parse the current line we read from the magic file.
		/// </summary>
		/// <param name="vs_line">the current line being read from the magic file</param>
		private void parse(String vs_line)
		{
			if (vs_line == null)
				throw new ArgumentException("Parsed Line is null and shouldn't be! Error parsing Magic File");

			// filter out comments and blank lines
            if (!vs_line.StartsWith("#") && vs_line.Trim().Length > 0)
			{
                MagicFileRecord o_mfr = null;
                if (!vs_line.StartsWith(">") && !vs_line.StartsWith("&"))
                {
                    // this is a new record
                    o_mfr = MagicFileRecord.createRecord(vs_line);
                    if (o_mfr != null)
                        IO_RECORDS.Add(o_mfr);
                }
                else
                {
                    // this is a continuation line for a sub record
                    // this should never happen but we need to make sure
                    if (o_mfr != null)
                    {
                        Controller.displayErrorMessage(vs_line);
                    }
                }
			}
		}
	}

	/// <summary>
	/// An object representation of a line from the MagicFile
	/// </summary>
	internal class MagicFileRecord
	{	
		// required mapping between the comparitor types and their literal string values
		private static IDictionary<MagicFileRecord.CompareType, String> XO_COMPARITOR_MAP = new Dictionary<MagicFileRecord.CompareType, String>();

        // the message to correspond to the type of file found
		private string s_message;
        internal string Message
        {
            get { return s_message; }
            //set { s_message = value;}
        }

        // the type of record this is
		private RecordType e_rtype = RecordType.BYTE;
        internal RecordType Type
        {
            get { return e_rtype; }
        }
		
        // the test value to look for
		private string s_string_test;
        internal string SearchString 
        {
            get { return s_string_test; }
        }
		
        // another test value that could exist
        private byte by_byte_test;
        internal byte SearchByte
        {
            get { return this.by_byte_test; }
        }

        // another test value that could exist
		private long l_number_test;
        internal long SearchLong 
        {
            get { return l_number_test; }
        }
        private ulong ul_number_test;
        internal ulong SearchULong
        {
            get { return ul_number_test; }
        }
		
        // another test value that could exist
        private int n_number_test;
        internal int SearchShort
        {
            get { return this.n_number_test; }
        }
        private uint un_number_test = 0;
        internal uint SearchUShort
        {
            get { return this.un_number_test; }
        }

        // another test value that could exis
		private DateTime o_date_test = new DateTime();
        internal DateTime SearchDate
        {
            get { return o_date_test; }
        }
        
        // more specific records to check
        private bool b_has_subrecords;
        internal bool HasSubRecords
        {
            get { return b_has_subrecords; }
        }

        // regex to search for
        private Regex io_regex;
        internal Regex SearchRegex
        {
            get { return io_regex; }
        }

        // get the comparitor, implied is =
		private CompareType io_comp_type = CompareType.EQ;
        internal CompareType ComparitorType
        {
            get { return io_comp_type; }
        }

        /// <summary>
        /// Need to prevent public instantiation so we use the 
        /// internal scope.
        /// </summary>
        private MagicFileRecord()
        {
			if (XO_COMPARITOR_MAP.Count < 1)
			{
				// prefill the comparitor type map
				XO_COMPARITOR_MAP.Add(MagicFileRecord.CompareType.ANY, "x");
				XO_COMPARITOR_MAP.Add(MagicFileRecord.CompareType.CLEAR_BITS, "^");
				XO_COMPARITOR_MAP.Add(MagicFileRecord.CompareType.EQ, "=");
				XO_COMPARITOR_MAP.Add(MagicFileRecord.CompareType.GT, ">");
				XO_COMPARITOR_MAP.Add(MagicFileRecord.CompareType.LT, "<");
				XO_COMPARITOR_MAP.Add(MagicFileRecord.CompareType.SET_BITS, "&");
                XO_COMPARITOR_MAP.Add(MagicFileRecord.CompareType.NOT, "!");
			}
        }

        /// <summary>
        /// Magic file constructor which takes a record string
        /// from the magic file and parses it into a usable record format
        /// </summary>
        /// <param name="vs_record">the record line read from the magic file</param>
        /// <exception cref="ArgumentException">thrown if the string record is null</exception>
        internal static MagicFileRecord createRecord(string vs_record)
        {
            if (vs_record == null || vs_record.Trim().Length == 0)
                throw new ArgumentException("Record String is null or empty!");
            
            // record to populate
            MagicFileRecord o_rec = new MagicFileRecord();
            
            // default pointer location into the array we will create
            int n_pointer = 1;

            // array of pieces to parse
            string[] as_pieces = vs_record.Split(null);
            as_pieces = MagicFileRecord.cleanArray(as_pieces);

            // flag to prevent parsing if there is an error condition
            bool b_parse_error = false;

            // extra value, if there is one, that should be ANDed with the next value in the array
            string s_and_value;            

            // this will do an enum assignment assuming we've isolated the enum
            Type o_type = typeof(RecordType);
            if (Enum.IsDefined(o_type, as_pieces[n_pointer]))
            {
                // this will only work when the = is implied
                o_rec.e_rtype = (RecordType)Enum.Parse(typeof(RecordType), as_pieces[n_pointer], true);
            }
            else
            {
                bool b_found_value = false;
                // this means we either have to parse out the type
                // or there is no type parse out and we need to exit
                foreach (RecordType rt in Enum.GetValues(o_type))
                    if (as_pieces[n_pointer].StartsWith(rt.ToString(), StringComparison.CurrentCultureIgnoreCase))
                    {
                        int n_idx = as_pieces[n_pointer].IndexOf(rt.ToString(), StringComparison.CurrentCultureIgnoreCase);
                        s_and_value = as_pieces[n_pointer].Substring(n_idx + rt.ToString().Length);
                        o_rec.e_rtype = rt;
                        b_found_value = true;
                        break;
                    }
                // if we didnt find a valid type of record, we need to stop parsing
                b_parse_error = !b_found_value;
            }

            // increase our index pointer by one
            n_pointer++;

            if( !b_parse_error )
            {
				// find the comparitor we need to use and store that off. remove it from the number
				// if there is one to remove, otherwise continue on
				CompareType e_ct = CompareType.EQ;
				foreach (CompareType ct in Enum.GetValues(typeof(CompareType)))
				{
					if (as_pieces[n_pointer].StartsWith(XO_COMPARITOR_MAP[ct]))
					{
						e_ct = ct;
						as_pieces[n_pointer] = as_pieces[n_pointer].Remove(0, 1);
						break;
					}
				}
				o_rec.io_comp_type = e_ct;

                // trim off any whitespace on the long number
                as_pieces[n_pointer] = as_pieces[n_pointer].Trim();

                // based on the type assign the next portion
                switch (o_rec.Type)
                {
                    case RecordType.BEDATE:
                        break;
                    case RecordType.BELONG:
                        break;
                    case RecordType.BESHORT:
                        break;

                    case RecordType.UBYTE:
                    case RecordType.BYTE:
                        try
                        {
                            o_rec.by_byte_test = Convert.ToByte(as_pieces[n_pointer], 16);
                        }
                        catch (Exception e)
                        { 
                            if (!Byte.TryParse(as_pieces[n_pointer], out o_rec.by_byte_test))
                            {
                                Controller.displayExcepton("Unable to convert to byte " + as_pieces[n_pointer] + " in " + vs_record, e);
                                b_parse_error = true;
                            }
                        }
                        break;

                    case RecordType.LEDATE:
                    case RecordType.DATE:
                        if (!DateTime.TryParse(as_pieces[n_pointer], out o_rec.o_date_test))
                        {
                            Controller.displayErrorMessage("Unable to parse date in: " + vs_record);
                            b_parse_error = true;
                        }
                        break;
                    case RecordType.LDATE:
                        break;
                    case RecordType.LELDATE:
                        break;
                        
                    case RecordType.UBELONG:
                        break;

                    case RecordType.ULELONG:
                    case RecordType.ULONG:
                        try
                        {
                            o_rec.ul_number_test = Convert.ToUInt64(as_pieces[n_pointer], 16);
                        }
                        catch (Exception)
                        {
                            if (!ulong.TryParse(as_pieces[n_pointer], out o_rec.ul_number_test))
                            {
                                Controller.displayErrorMessage("Unable to parse: " + vs_record);
                                b_parse_error = true;
                            }
                        }
                        break;
                    case RecordType.LELONG:
                    case RecordType.LONG:
                        // this is a hack, but some of the hex string longs end in L
                        // which C# does not understand so we need to remove the L

                        if (as_pieces[n_pointer].EndsWith("L"))
                            as_pieces[n_pointer] = as_pieces[n_pointer].Remove(as_pieces[n_pointer].Length - 1, 1);

                        try
                        {
                            o_rec.l_number_test = Convert.ToInt64(as_pieces[n_pointer], 16);
                        }
                        catch (Exception)
                        {
							try
							{
								o_rec.l_number_test = long.Parse(as_pieces[n_pointer], System.Globalization.NumberStyles.HexNumber);
							}
							catch (Exception vo_e2)
							{
                                Controller.displayExcepton("Cannot convert long in record " + vs_record, vo_e2);
								b_parse_error = true;
							}
                        }
                        break;

                    case RecordType.REGEX:
                        try
                        {
                            Regex o_regex = new Regex(as_pieces[n_pointer] , RegexOptions.IgnoreCase & RegexOptions.IgnorePatternWhitespace);
                            o_rec.io_regex = o_regex;
                        }
                        catch (Exception vo_e)
                        {
                            Controller.displayExcepton("Cannot convert regex in line " + vs_record, vo_e);
                            b_parse_error = true;
                        }
                        break;

                    case RecordType.SEARCH:
                        break;
                    
					case RecordType.ULESHORT:
                    case RecordType.USHORT:
                        try
                        {
                            o_rec.un_number_test = Convert.ToUInt32(as_pieces[n_pointer], 16);
                        }
                        catch (Exception e)
                        {
                            try
                            {
                                o_rec.un_number_test = Convert.ToUInt32(as_pieces[n_pointer]);
                            }
                            catch (Exception)
                            {
                                Controller.displayExcepton("Cannot convert ushort in line " + vs_record, e);
                                b_parse_error = true;
                            }
                        }
                        break;
                    case RecordType.LESHORT:
                    case RecordType.SHORT:
                        try
                        {
                            o_rec.n_number_test = Convert.ToInt32(as_pieces[n_pointer], 16);
                        }
                        catch (Exception )
                        {
                            try
                            {
                                o_rec.n_number_test = Convert.ToInt32(as_pieces[n_pointer]);
                            }
                            catch (Exception e2)
                            {
                                Controller.displayExcepton("Cannot parse short in line " + vs_record, e2);
                                b_parse_error = true;
                            }
                        }
                        break;

                    case RecordType.USTRING:
                    case RecordType.STRING:
                        o_rec.s_string_test = as_pieces[n_pointer];
                        break;
                    default:
                        Controller.displayErrorMessage("Unknown Record Type: " + o_rec.e_rtype);
                        b_parse_error = true;
                        break;
                }
                // advance one for the switch statemtn since that is required regardless of type of record
                n_pointer++;

                // set the message type to the fourth value in the array
				if (!b_parse_error)
				{
					StringBuilder o_sb = new StringBuilder("INFO:");
					for (int i = n_pointer; i < as_pieces.Length; i++)
						o_sb.Append(" " + as_pieces[i]);
					o_rec.s_message = o_sb.ToString();
				}
            }

            if (b_parse_error)
                o_rec = null;
            return o_rec;
        }

        /// <summary>
        /// Returns a String representation of this record
        /// </summary>
        /// <returns>String</returns>
        public override string ToString()
        {
            StringBuilder o_sb = new StringBuilder("MagicFileRecord: ");
            o_sb.Append(this.Type.ToString());
            o_sb.Append(" " + this.Message);
            return o_sb.ToString();
        }

        /// <summary>
        /// Removes empty strings and null elements from the array of strings passed in
        /// </summary>
        /// <param name="vas_pieces">array of strings</param>
        /// <returns>clean array of strings</returns>
        private static string[] cleanArray(string[] vas_pieces)
        {
            if (vas_pieces == null || vas_pieces.Length < 1)
                throw new ArgumentException("Array is null or contains no elements!");
            ICollection<string> o_pieces = new LinkedList<string>();
            foreach (string s in vas_pieces)
                if (s != null && s.Trim().Length > 0)
                    o_pieces.Add(s);
            return o_pieces.ToArray<string>();
        }

		/// <summary>
		/// The type of comparison that must be done given the type of 
		/// </summary>
		internal enum CompareType
		{
			// requires that the value specificed be greater than the values in the file (<)
			LT,
			// requires that the value specified be less than the values in the file (>)
			GT,
			// requires that all of hte values in the file match this one (=)
			EQ,
			// requires that all of the bits for the given number of bytes have these bits set (&)
			SET_BITS,
			// requires that all of the bits that are set (^)
			CLEAR_BITS,
			// requires any bits to match (x)
			ANY,
            // not - required that the byte NOT equal a certain thing
            NOT
		}

		/// <summary>
		/// The different types of records contained in the magic file. The positive values indicate the number
		/// of bytes reprented that must be parsed when searching. If the value is a -1, then the number of bytes don't matter,
		/// and one has to search based on the value of the pieces of the rest of the record.
		/// </summary>
		internal enum RecordType
		{
			BYTE,
            UBYTE,
			SHORT,
            USHORT,
			LONG,
            ULONG,
			STRING,
            USTRING,
			DATE,
			/// <summary>
			/// local date instead of UTC
			/// </summary>
			LDATE,
			/// <summary>
			/// big endian ordered short
			/// </summary>
			BESHORT,
            UBESHORT,
			/// <summary>
			/// big endian ordered long
			/// </summary>
			BELONG,
            UBELONG,
			/// <summary>
			/// big endian ordered date
			/// </summary>
			BEDATE,
			/// <summary>
			/// little endian ordered short
			/// </summary>
			LESHORT,
            ULESHORT,
			/// <summary>
			/// little endian ordered long
			/// </summary>
			LELONG,
            ULELONG,
			/// <summary>
			/// little endian ordered date
			/// </summary>
			LEDATE,
			/// <summary>
			/// little endian ordered local date
			/// </summary>
			LELDATE,
			/// <summary>
			/// A specific regular expression to match
			/// </summary>
			REGEX,
			/// <summary>
			/// A specific search string to match
			/// </summary>
			SEARCH
		}
	}
}
